#include "hlx_daemon_packet_reader.hh"
#include <sys/socket.h>
#include <features.h>    /* for the glibc version number */
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1
#include <netpacket/packet.h>
#include <net/ethernet.h>     /* the L2 protocols */
#else
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>   /* The L2 protocols */
#endif
#include <netinet/in.h>
#include <unistd.h>
#include <sys/poll.h>

class hlx_daemon_packet_reader_impl {
public:
  int packet_socket;
  int timeout;
};

hlx_daemon_packet_reader::hlx_daemon_packet_reader() {
  m_impl=new hlx_daemon_packet_reader_impl;
  m_impl->packet_socket=socket(PF_INET, SOCK_DGRAM, 0);
  m_impl->timeout=1000*1000; // 1 second

  struct sockaddr_in tobind;
  tobind.sin_family=AF_INET;
  tobind.sin_port=htons(21306);
  tobind.sin_addr.s_addr=INADDR_ANY;

  bind(m_impl->packet_socket,(sockaddr*)(&tobind),sizeof(tobind));

}

hlx_daemon_packet_reader::~hlx_daemon_packet_reader() {
  close(m_impl->packet_socket);
  delete m_impl;
}

void hlx_daemon_packet_reader::setTimeout(int us) {
  m_impl->timeout=us;
}

void hlx_daemon_packet_reader::read(std::vector<hlx_daemon_packet*>& pkts) {
  
  pkts.clear();
  pollfd pollstruct;
  pollstruct.fd=m_impl->packet_socket;
  pollstruct.events=1;
  int total=0;
  do {
    if (poll(&pollstruct,1,m_impl->timeout/1000)<=0) return;


    pkts.push_back(new hlx_daemon_packet);
  
    int igot=recvfrom(m_impl->packet_socket,pkts.back(),sizeof(hlx_daemon_packet), MSG_TRUNC, 0, 0);

    printf("I got %d\n",igot);
    total++;
  } while (total<10);

}
